
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
 
#define UINT32  unsigned long
#define UINT16  unsigned short
#define UINT8   unsigned char
 
// Declare variables and structures
HANDLE hSerial = INVALID_HANDLE_VALUE;
DCB dcbSerialParams = {0};
COMMTIMEOUTS timeouts = {0};
DWORD dwBytesWritten = 0;
char dev_name[MAX_PATH] = "";
int dev_number = -1;
int baudrate = 115200;
int id = -1;
UINT8 *inbuf;
FILE *f1;

 
void CloseSerialPort()
{
    if (hSerial != INVALID_HANDLE_VALUE)
    {
        fclose(f1);
        free(inbuf);
        // Close serial port
        fprintf(stderr, "\nClosing serial port...");
        if (CloseHandle(hSerial) == 0)
            fprintf(stderr, "Error\n");
        else fprintf(stderr, "OK\n");
    }
}
 
int main(int argc, char *argv[])
{
    // Parse command line arguments. Available options:
    // /comport DEVICE_NUMBER
    // /baudrate BITS_PER_SECOND
	
	const char *filename;
	unsigned int len, len2, com_number, i, j, crc_calc;
	char game_name[100];
    int state = 1;
    char c;
    UINT8 message_buffer[30];
    DWORD bytes_read, bytes_written, total_bytes_written;
   
    char reset_code[]={0xE0,0xFF,0x03,0xF0,0xD9,0xCB,0};
	char assign_code[]={0xE0,0xFF,0x03,0xF1,0x01,0xF4,0};
	char assign_reply[]={0xE0,0x00,0x03,0x01,0x01,0x05,0};
	char readid_code[]={0xE0,0x01,0x02,0x10,0x13,0}; //CMD_READID = 0x10,
	char formatversion_code[]={0xE0,0x01,0x02,0x11,0x14,0}; //CMD_FORMATVERSION = 0x11,
	char jvsversion_code[]={0xE0,0x01,0x02,0x12,0x15,0}; //CMD_JVSVERSION = 0x12,
	char commver_code[]={0xE0,0x01,0x02,0x13,0x16,0}; //CMD_COMMSVERSION = 0x13,
	char getfeatures_code[]={0xE0,0x01,0x02,0x14,0x17,0}; //CMD_GETFEATURES = 0x14,
	
   printf("(C)2016 JVS Information Reader by Darksoft v0.1\n");
   printf("In order to use it, you need to connect your JVS card to the COM port of your computer using a RS485<->RS232 adapter.\n\n");
   printf("Use at your complete own risk. Only for educational purposes.\n\n");
   if ((argc!=3))
   {
   printf("\nError in parameters!\n");
   printf("You need to enter the COM Port # as parameter1 and baud rate as parameter2\n");
   printf("For example: Read_JVS.exe 1 115200\n");
   exit(1);
   }
		com_number = atoi(argv[1]);
		if ((com_number<1)||(com_number>40)) printf("Error: no valid COM specified\n");
			
		baudrate = atoi(argv[2]);
		if ((baudrate<2600)||(baudrate>115200)) printf("Error: no baudrate value specified\n");

    printf("Using COM Port = COM%d\n", com_number);
    printf("Using Baudrate = %d\n", baudrate);

	inbuf = malloc(0x00010000);
	if (!inbuf)
		{
		printf("Unable to locate memory. Quitting!\n");
		exit(3);
    }	
	
	
    // Register function to close serial port at exit time
    atexit(CloseSerialPort);
 
 
        sprintf(dev_name, "\\\\.\\COM%d", com_number);
//        printf("Trying COM%d...", com_number);
//        hSerial = CreateFile(dev_name, GENERIC_READ|GENERIC_WRITE, 0, 0,
        hSerial = CreateFile("COM1", GENERIC_READ|GENERIC_WRITE, 0, NULL,
                                OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
        if (hSerial!=INVALID_HANDLE_VALUE)
        {
            printf("Com Port %d Open OK\n", com_number);
        }
        else 
		{   
			printf("Com Port %d Open FAILED\n", com_number);
			exit(1);
		}

 
    // If we get this far, a serial port was successfully opened
    fprintf(stderr, "Opening COM%d at %d bps\n\n", com_number, baudrate);
 
    // Set device parameters:
    dcbSerialParams.DCBlength = sizeof(dcbSerialParams);
    if (GetCommState(hSerial, &dcbSerialParams) == 0)
        printf("Error getting device state\n");
        
//    dcbSerialParams.BaudRate = baudrate;
    dcbSerialParams.BaudRate = CBR_115200;	
    dcbSerialParams.ByteSize = 8;
    dcbSerialParams.StopBits = ONESTOPBIT;
    dcbSerialParams.Parity = NOPARITY;
    
    if(SetCommState(hSerial, &dcbSerialParams) == 0)
        printf("Error setting device parameters\n");
 
    // Set COM port timeout settings
/*    timeouts.ReadIntervalTimeout = 0;
    timeouts.ReadTotalTimeoutConstant = 50;
    timeouts.ReadTotalTimeoutMultiplier = 0;
    timeouts.WriteTotalTimeoutConstant = 50;
    timeouts.WriteTotalTimeoutMultiplier = 0;
*/    
    COMMTIMEOUTS timeouts =  {MAXDWORD, 0, 0, 0, 15000};    
    
    if(SetCommTimeouts(hSerial, &timeouts) == 0)
        printf("Error setting timeouts\n");
 
sleep(40);	

    printf("Sending Reset...\n");
    if(!WriteFile(hSerial, reset_code, 6, &bytes_written, NULL))
    {
        printf("Error in 1st Reset\n");
        CloseHandle(hSerial);
        exit(1);
    }   
//    printf("%d bytes sent\n", bytes_written);
sleep(200);	

    printf("Sending Second Reset...\n");
    if(!WriteFile(hSerial, reset_code, 6, &bytes_written, NULL))
    {
        printf("Error in 2nd Reset\n");
        CloseHandle(hSerial);
        exit(1);
    }   
//    printf("%d bytes sent\n", bytes_written);
sleep(200);	
	

    printf("Sending Assign command...\n");
    if(!WriteFile(hSerial, assign_code, 6, &bytes_written, NULL))
    {
        printf("Error Sending Assign command\n");
        CloseHandle(hSerial);
        exit(1);
    }   
//    printf("%d bytes sent\n", bytes_written);

sleep(200);	
	
    
	ReadFile(hSerial, message_buffer, 6, &bytes_read, NULL);
		if (bytes_read != 6)
            {
                printf("Error in response. Check that JVS is ON and properly connected");
				CloseHandle(hSerial);
				exit(9);
            }
        if (!strcmp(message_buffer,assign_reply)==0) {
                printf("Error in response. JVS is on but not properly connected");
				CloseHandle(hSerial);
				exit(9);
            }

sleep(200);	
	


    printf("Sending ReadID command...\n");
    if(!WriteFile(hSerial, readid_code, 5, &bytes_written, NULL))
    {
        printf("Error Sending ReadID command\n");
        CloseHandle(hSerial);
        exit(1);
    }   
sleep(40);	
   
	ReadFile(hSerial, message_buffer, 100, &bytes_read, NULL);
        printf("Answer received of %08x bytes read\n", bytes_read);
        for (j=0;j<bytes_read;j++)
            {
            printf("%02x ",message_buffer[j]);
            }

            sleep(40);
                                                 

    printf("\nSending formatversion_code command...\n");
    if(!WriteFile(hSerial, formatversion_code, 5, &bytes_written, NULL))
    {
        printf("Error Sending formatversion_code command\n");
        CloseHandle(hSerial);
        exit(1);
    }   
sleep(40);	
    
	ReadFile(hSerial, message_buffer, 100, &bytes_read, NULL);
        printf("Answer received of %08x bytes read\n", bytes_read);
        for (j=0;j<bytes_read;j++)
            {
            printf("%02x ",message_buffer[j]);
            }

sleep(40);	

    printf("\nSending jvsversion_code command...\n");
    if(!WriteFile(hSerial, jvsversion_code, 5, &bytes_written, NULL))
    {
        printf("Error Sending jvsversion_code command\n");
        CloseHandle(hSerial);
        exit(1);
    }   
sleep(40);	
   
	ReadFile(hSerial, message_buffer, 100, &bytes_read, NULL);
        printf("Answer received of %08x bytes read\n", bytes_read);
        for (j=0;j<bytes_read;j++)
            {
            printf("%02x ",message_buffer[j]);
            }

sleep(40);	

    printf("\nSending commver_code command...\n");
    if(!WriteFile(hSerial, commver_code, 5, &bytes_written, NULL))
    {
        printf("Error Sending commver_code command\n");
        CloseHandle(hSerial);
        exit(1);
    }   
sleep(40);	
    
	ReadFile(hSerial, message_buffer, 100, &bytes_read, NULL);
        printf("Answer received of %08x bytes read\n", bytes_read);
        for (j=0;j<bytes_read;j++)
            {
            printf("%02x ",message_buffer[j]);
            }

sleep(40);	

    printf("\nSending getfeatures_code command...\n");
    if(!WriteFile(hSerial, getfeatures_code, 5, &bytes_written, NULL))
    {
        printf("Error Sending getfeatures_code command\n");
        CloseHandle(hSerial);
        exit(1);
    }   
sleep(40);	
    
	ReadFile(hSerial, message_buffer, 100, &bytes_read, NULL);
        printf("Answer received of %08x bytes read\n", bytes_read);
        for (j=0;j<bytes_read;j++)
            {
            printf("%02x ",message_buffer[j]);
            }

sleep(40);	

	CloseHandle(hSerial);
	printf("\nReading done! \n");
	free(inbuf);
    return 0;
}
